#!/usr/bin/python
# -*- utf-8 -*-

# see: http://theory.stanford.edu/~amitp/GameProgramming/

import heapq

inf = float('inf')

# -------------------------------------------------------------
#  Priority Queue implementation
# -------------------------------------------------------------

class PriorityQueue(object):
    def __init__(self, lst=None):
        self._index = dict()
        self._data = list()

    def __len__(self):
        return len(self._data)

    def __contains__(self, obj):
        return (obj in self._index)

    def push(self, prio, obj):
        heapq.heappush(self._data, (prio, obj))
        self._index[obj] = prio

    def pop(self):
        prio, obj = heapq.heappop(self._data)
        del self._index[obj]
        return prio, obj

    def remove(self, obj):
        prio = self._index[obj]
        del self._index[obj]
        self._data.remove((prio, obj))
        heapq.heapify(self._data)

# -------------------------------------------------------------
#  A* Algorithm on generic networks
# -------------------------------------------------------------

def reconstructPath(startnode, endnode):
    path = list()
    path.insert(0, endnode)
    node = endnode
    while not node.parent is None:
        node = node.parent
        path.insert(0, node)
    return path

def calcCost(hfunc, node, endnode):
    if node.cost == inf:
        node.cost = hfunc(node, endnode)
    return node.cost

def aStar(nodes, startnode, endnode, hfunc):
    for n in nodes:
        n.cost = inf
        n.parent = None

    queue = PriorityQueue()
    queue.push(0, startnode)
    closed = dict()
    i = 0
    while len(queue) > 0:
        i += 1
        f, node = queue.pop()
        if node == endnode:
            return reconstructPath(startnode, endnode)
        closed[node] = node
        calcCost(hfunc, node, endnode)
#        print "step %d:" % i, repr(node), node.cost
        j = 0
        for edge in node.edges:
            j += 1
#            print "  %d.%d:" % (i, j), edge
            if edge.from_node == node:
                neighbor = edge.to_node
            else:
                neighbor = edge.from_node
            calcCost(hfunc, node, endnode)
            cost = node.cost + edge.weight
            if neighbor in queue and (neighbor.cost < node.cost):
                queue.remove(neighbor)
            if neighbor in closed and (cost < neighbor.cost):
                del closed[neighbor]
            if not neighbor in queue and not neighbor in closed:
                neighbor.cost = cost
                queue.push(cost, neighbor)
                neighbor.parent = node

# -------------------------------------------------------------
#  
# -------------------------------------------------------------

