#!/usr/bin/python
# -*- utf-8 -*-

import sys
import math
import osm.parse
import osm.net
import osm.route

def findnode(nodes, nodeid):
    for n in nodes:
        if n.node == nodeid:
            return n
    return None

class Heuristic(object):
    def __init__(self, nodes):
        self.nodes = nodes

    def __call__(self, node, endnode):
        n1 = self.nodes[node.node]
        n2 = self.nodes[endnode.node]
        return math.hypot(abs(n1.lat - n2.lat),
                          abs(n1.lon - n2.lon))

if __name__ == '__main__':
    startnodeid = int(sys.argv[2])
    endnodeid = int(sys.argv[3])

    print " - parsing"
    parser = osm.parse.OsmParser()
    f = file(sys.argv[1], 'r')
    parser.parseFile(f)
    f.close()

    print " - create simplified net"
    nodes, edges = osm.net.buildNet(parser, startnodeid)

    startnode = findnode(nodes, startnodeid)
    endnode = findnode(nodes, endnodeid)

    # calculate route with simplified network
    print " - routing A*"
    hfunc = Heuristic(parser.nodes)
    route = osm.route.aStar(nodes, startnode, endnode, hfunc)

    # create path from route
    print " - create path"
    path = osm.net.buildPath(parser.nodes, route)

    # create result route path OSM file
    print " - save path to route.osm"
    nl = list()
    nid = -1
    way = osm.parse.Way(nid)
    for p in path:
        nid -= 1
        n = osm.parse.Node(nid, p.lat, p.lon)
        n.tag['created_by'] = 'PyOSM'
        way.nodes.append(n)
        nl.append(n)
    f = file('route.osm', 'w')
    osm.parse.createXML(f, nl, [way], [])
    f.close()
